Implement support for the Winbond W83627THG.

Signed-off-by: Stefan Reinauer <stepan@coresystems.de>

Index: src/superio/winbond/w83627thg/w83627thg_early_serial.c
===================================================================
--- src/superio/winbond/w83627thg/w83627thg_early_serial.c	(revision 0)
+++ src/superio/winbond/w83627thg/w83627thg_early_serial.c	(revision 0)
@@ -0,0 +1,25 @@
+#include <arch/romcc_io.h>
+#include "w83627thg.h"
+
+static inline void pnp_enter_ext_func_mode(device_t dev)
+{
+	unsigned int port = dev >> 8;
+	outb(0x87, port);
+	outb(0x87, port);
+}
+
+static void pnp_exit_ext_func_mode(device_t dev)
+{
+	unsigned int port = dev >> 8;
+	outb(0xaa, port);
+}
+
+static void w83627thg_enable_serial(device_t dev, unsigned int iobase)
+{
+	pnp_enter_ext_func_mode(dev);
+	pnp_set_logical_device(dev);
+	pnp_set_enable(dev, 0);
+	pnp_set_iobase(dev, PNP_IDX_IO0, iobase);
+	pnp_set_enable(dev, 1);
+	pnp_exit_ext_func_mode(dev);
+}
Index: src/superio/winbond/w83627thg/Config.lb
===================================================================
--- src/superio/winbond/w83627thg/Config.lb	(revision 0)
+++ src/superio/winbond/w83627thg/Config.lb	(revision 0)
@@ -0,0 +1,2 @@
+config chip.h
+object superio.o
Index: src/superio/winbond/w83627thg/superio.c
===================================================================
--- src/superio/winbond/w83627thg/superio.c	(revision 0)
+++ src/superio/winbond/w83627thg/superio.c	(revision 0)
@@ -0,0 +1,109 @@
+/* Copyright 2000  AG Electronics Ltd. */
+/* Copyright 2003-2004 Linux Networx */
+/* Copyright 2004 Tyan 
+   By LYH change from PC87360 */
+/* This code is distributed without warranty under the GPL v2 (see COPYING) */
+
+#include <arch/io.h>
+#include <device/device.h>
+#include <device/pnp.h>
+#include <console/console.h>
+#include <string.h>
+#include <bitops.h>
+#include <uart8250.h>
+#include <pc80/keyboard.h>
+#include "chip.h"
+#include "w83627thg.h"
+
+static void w83627thg_enter_ext_func_mode(device_t dev) 
+{
+        outb(0x87, dev->path.u.pnp.port);
+        outb(0x87, dev->path.u.pnp.port);
+}
+static void w83627thg_exit_ext_func_mode(device_t dev) 
+{
+        outb(0xaa, dev->path.u.pnp.port);
+}
+
+static void w83627thg_init(device_t dev)
+{
+	struct superio_winbond_w83627thg_config *conf;
+	struct resource *res0, *res1;
+	/* Wishlist handle well known programming interfaces more
+	 * generically.
+	 */
+	if (!dev->enabled) {
+		return;
+	}
+	conf = dev->chip_info;
+	switch(dev->path.u.pnp.device) {
+	case W83627THG_SP1: 
+		res0 = find_resource(dev, PNP_IDX_IO0);
+		init_uart8250(res0->base, &conf->com1);
+		break;
+	case W83627THG_SP2:
+		res0 = find_resource(dev, PNP_IDX_IO0);
+		init_uart8250(res0->base, &conf->com2);
+		break;
+	case W83627THG_KBC:
+		res0 = find_resource(dev, PNP_IDX_IO0);
+		res1 = find_resource(dev, PNP_IDX_IO1);
+		init_pc_keyboard(res0->base, res1->base, &conf->keyboard);
+		break;
+	}
+}
+
+static void w83627thg_set_resources(device_t dev)
+{
+	w83627thg_enter_ext_func_mode(dev);
+	pnp_set_resources(dev);
+	w83627thg_exit_ext_func_mode(dev);
+}
+
+static void w83627thg_enable_resources(device_t dev)
+{
+	w83627thg_enter_ext_func_mode(dev);
+	pnp_enable_resources(dev);
+	w83627thg_exit_ext_func_mode(dev);
+}
+
+static void w83627thg_enable(device_t dev)
+{
+	w83627thg_enter_ext_func_mode(dev);   
+	pnp_enable(dev);
+	w83627thg_exit_ext_func_mode(dev);  
+}
+
+
+static struct device_operations ops = {
+	.read_resources   = pnp_read_resources,
+	.set_resources    = w83627thg_set_resources,
+	.enable_resources = w83627thg_enable_resources,
+	.enable           = w83627thg_enable,
+	.init             = w83627thg_init,
+};
+
+static struct pnp_info pnp_dev_info[] = {
+        { &ops, W83627THG_FDC,  PNP_IO0 | PNP_IRQ0 | PNP_DRQ0, { 0x07f8, 0}, },
+        { &ops, W83627THG_PP,   PNP_IO0 | PNP_IRQ0 | PNP_DRQ0, { 0x07f8, 0}, },
+        { &ops, W83627THG_SP1,  PNP_IO0 | PNP_IRQ0, { 0x7f8, 0 }, },
+        { &ops, W83627THG_SP2,  PNP_IO0 | PNP_IRQ0, { 0x7f8, 0 }, },
+        // No 4 { 0,},
+        { &ops, W83627THG_KBC,  PNP_IO0 | PNP_IO1 | PNP_IRQ0 | PNP_IRQ1, { 0x7ff, 0 }, { 0x7ff, 0x4}, },
+        { &ops, W83627THG_GAME_MIDI_GPIO1, PNP_IO0 | PNP_IO1 | PNP_IRQ0, { 0x7ff, 0 }, {0x7fe, 4} },
+        { &ops, W83627THG_GPIO2,},
+        { &ops, W83627THG_GPIO3,},
+        { &ops, W83627THG_ACPI, PNP_IRQ0,  },
+        { &ops, W83627THG_HWM,  PNP_IO0 | PNP_IRQ0, { 0xff8, 0 } },
+};
+
+static void enable_dev(device_t dev)
+{
+	pnp_enable_devices(dev, &ops,
+		sizeof(pnp_dev_info)/sizeof(pnp_dev_info[0]), pnp_dev_info);
+}
+
+struct chip_operations superio_winbond_w83627thg_ops = {
+	CHIP_NAME("Winbond W83627THG Super I/O")
+	.enable_dev = enable_dev,
+};
Index: src/superio/winbond/w83627thg/chip.h
===================================================================
--- src/superio/winbond/w83627thg/chip.h	(revision 0)
+++ src/superio/winbond/w83627thg/chip.h	(revision 0)
@@ -0,0 +1,9 @@
+#include <pc80/keyboard.h>
+#include <uart8250.h>
+
+extern struct chip_operations superio_winbond_w83627thg_ops;
+
+struct superio_winbond_w83627thg_config {
+	struct uart8250 com1, com2;
+	struct pc_keyboard keyboard;
+};
Index: src/superio/winbond/w83627thg/w83627thg.h
===================================================================
--- src/superio/winbond/w83627thg/w83627thg.h	(revision 0)
+++ src/superio/winbond/w83627thg/w83627thg.h	(revision 0)
@@ -0,0 +1,10 @@
+#define W83627THG_FDC              0   /* Floppy */
+#define W83627THG_PP               1   /* Parallel Port */
+#define W83627THG_SP1              2   /* Com1 */
+#define W83627THG_SP2              3   /* Com2 */
+#define W83627THG_KBC              5   /* Keyboard & Mouse */
+#define W83627THG_GAME_MIDI_GPIO1  7
+#define W83627THG_GPIO2            8
+#define W83627THG_GPIO3            9
+#define W83627THG_ACPI            10
+#define W83627THG_HWM             11   /* Hardware Monitor */
